@@ -20,11 +20,17 @@
 package org.apache.cayenne.access;
 
 import org.apache.cayenne.Cayenne;
+import org.apache.cayenne.ObjectContext;
 import org.apache.cayenne.PersistenceState;
+import org.apache.cayenne.access.translator.select.DefaultSelectTranslator;
+import org.apache.cayenne.configuration.server.ServerRuntime;
 import org.apache.cayenne.di.Inject;
-import org.apache.cayenne.exp.ExpressionFactory;
+import org.apache.cayenne.exp.property.PropertyFactory;
+import org.apache.cayenne.map.DefaultEntityResultSegment;
+import org.apache.cayenne.query.ColumnSelect;
 import org.apache.cayenne.query.EJBQLQuery;
-import org.apache.cayenne.query.SelectQuery;
+import org.apache.cayenne.query.ObjectSelect;
+import org.apache.cayenne.query.SelectById;
 import org.apache.cayenne.reflect.PersistentDescriptor;
 import org.apache.cayenne.test.jdbc.DBHelper;
 import org.apache.cayenne.test.jdbc.TableHelper;
@@ -35,6 +41,7 @@ import org.apache.cayenne.testdo.testmap.Gallery;
 import org.apache.cayenne.unit.di.server.CayenneProjects;
 import org.apache.cayenne.unit.di.server.ServerCase;
 import org.apache.cayenne.unit.di.server.UseServerRuntime;
+import org.junit.Before;
 import org.junit.Test;
 
 import java.math.BigDecimal;
@@ -50,17 +57,26 @@ import static org.junit.Assert.assertTrue;
 @UseServerRuntime(CayenneProjects.TESTMAP_PROJECT)
 public class DataContextFlattenedAttributesIT extends ServerCase {
 
+    @Inject
+    private ServerRuntime runtime;
+
     @Inject
     private DataContext context;
 
     @Inject
     private DBHelper dbHelper;
 
-    private void createTestDataSet() throws Exception {
-        TableHelper tArtist = new TableHelper(dbHelper, "ARTIST");
+    private TableHelper tArtist;
+    private TableHelper tPainting;
+    private TableHelper tPaintingInfo;
+    private TableHelper tGallery;
+
+    @Before
+    public void createTestDataSetStructure() {
+        tArtist = new TableHelper(dbHelper, "ARTIST");
         tArtist.setColumns("ARTIST_ID", "ARTIST_NAME", "DATE_OF_BIRTH");
 
-        TableHelper tPainting = new TableHelper(dbHelper, "PAINTING");
+        tPainting = new TableHelper(dbHelper, "PAINTING");
         tPainting.setColumns(
                 "PAINTING_ID",
                 "PAINTING_TITLE",
@@ -73,12 +89,14 @@ public class DataContextFlattenedAttributesIT extends ServerCase {
                 Types.DECIMAL,
                 Types.INTEGER);
 
-        TableHelper tPaintingInfo = new TableHelper(dbHelper, "PAINTING_INFO");
+        tPaintingInfo = new TableHelper(dbHelper, "PAINTING_INFO");
         tPaintingInfo.setColumns("PAINTING_ID", "TEXT_REVIEW");
 
-        TableHelper tGallery = new TableHelper(dbHelper, "GALLERY");
+        tGallery = new TableHelper(dbHelper, "GALLERY");
         tGallery.setColumns("GALLERY_ID", "GALLERY_NAME");
+    }
 
+    private void createTestDataSet() throws Exception {
         long dateBase = System.currentTimeMillis();
         for (int i = 1; i <= 4; i++) {
             tArtist.insert(i + 1, "artist" + i, new java.sql.Date(dateBase
@@ -111,12 +129,11 @@ public class DataContextFlattenedAttributesIT extends ServerCase {
     @Test
     public void testSelectCompound1() throws Exception {
         createTestDataSet();
-        SelectQuery<CompoundPainting> query = SelectQuery.query(CompoundPainting.class);
-        List<CompoundPainting> objects = query.select(context);
+        List<CompoundPainting> objects = ObjectSelect.query(CompoundPainting.class).select(context);
 
         assertNotNull(objects);
         assertEquals(8, objects.size());
-        assertTrue("CompoundPainting expected, got null", objects.get(0) != null);
+        assertNotNull("CompoundPainting expected, got null", objects.get(0));
 
         for (CompoundPainting painting : objects) {
             Number id = (Number) painting.getObjectId().getIdSnapshot().get("PAINTING_ID");
@@ -153,10 +170,9 @@ public class DataContextFlattenedAttributesIT extends ServerCase {
     @Test
     public void testSelectCompound2() throws Exception {
         createTestDataSet();
-        SelectQuery<CompoundPainting> query = SelectQuery.query(
-                CompoundPainting.class,
-                ExpressionFactory.matchExp("artistName", "artist2"));
-        List<CompoundPainting> objects = query.select(context);
+
+        List<CompoundPainting> objects = ObjectSelect.query(CompoundPainting.class, CompoundPainting.ARTIST_NAME.eq("artist2"))
+                .select(context);
 
         assertNotNull(objects);
         assertEquals(2, objects.size());
@@ -185,12 +201,120 @@ public class DataContextFlattenedAttributesIT extends ServerCase {
     @Test
     public void testSelectCompoundLongNames() throws Exception {
         createTestDataSet();
-        SelectQuery<CompoundPaintingLongNames> query = SelectQuery.query(CompoundPaintingLongNames.class);
         // the error was thrown on query execution
-        List<?> objects = context.performQuery(query);
+        List<?> objects = ObjectSelect.query(CompoundPaintingLongNames.class).select(context);
         assertNotNull(objects);
     }
 
+    @Test
+    public void testColumnQueryWithFlattenedAttribute() throws Exception {
+        createTestDataSet();
+        ColumnSelect<Object[]> originalQuery = ObjectSelect.query(CompoundPaintingLongNames.class)
+                .columns(PropertyFactory.createSelf(CompoundPaintingLongNames.class));
+
+        DataNode dataNode = context.getParentDataDomain().getDataNodes().iterator().next();
+        DefaultSelectTranslator translator =
+                new DefaultSelectTranslator(originalQuery, dataNode.getAdapter(), context.getEntityResolver());
+
+        translator.getSql();
+
+        DefaultEntityResultSegment segment = (DefaultEntityResultSegment) originalQuery
+                .getMetaData(context.getEntityResolver())
+                .getResultSetMapping()
+                .get(0);
+
+        assertEquals(12, segment.getFields().size());
+        assertEquals(12, translator.getResultColumns().length);
+        assertEquals(segment.getFields().size(), translator.getResultColumns().length);
+    }
+
+    @Test
+    public void testSelectColumnQuery() throws Exception {
+        createTestDataWithDeletion();
+
+        ColumnSelect<CompoundPaintingLongNames> originalQuery = ObjectSelect.query(CompoundPaintingLongNames.class)
+                .column(PropertyFactory.createSelf(CompoundPaintingLongNames.class));
+
+        CompoundPaintingLongNames beforeCompoundPainting = originalQuery
+                .where(CompoundPaintingLongNames.PAINTING_ID_PK_PROPERTY.eq(1))
+                .selectOne(context);
+
+        String beforeArtistNameFromContext = beforeCompoundPainting.getArtistLongName();
+        String beforePaintingTitleFromContext = beforeCompoundPainting.getPaintingTitle();
+
+        String beforeArtistNameFromDatabase = (String) tArtist.selectAll().get(0)[1];
+        String beforePaintingTitleFromDatabase = (String) tPainting.selectAll().get(0)[1];
+
+        assertNotNull(beforeArtistNameFromDatabase);
+        assertNotNull(beforePaintingTitleFromDatabase);
+
+        assertEquals(beforeArtistNameFromDatabase.trim(), beforeArtistNameFromContext);
+        assertEquals(beforePaintingTitleFromDatabase.trim(), beforePaintingTitleFromContext);
+
+        beforeCompoundPainting.setArtistLongName("some");
+        beforeCompoundPainting.setPaintingTitle("omes");
+        context.commitChanges();
+
+        CompoundPaintingLongNames afterCompoundPainting = originalQuery
+                .where(CompoundPaintingLongNames.PAINTING_ID_PK_PROPERTY.eq(1))
+                .selectOne(context);
+
+        String afterArtistNameFromContext = afterCompoundPainting.getArtistLongName();
+        String afterPaintingTitleFromContext = afterCompoundPainting.getPaintingTitle();
+
+        String afterArtistNameFromDatabase = (String) tArtist.selectAll().get(0)[1];
+        String afterPaintingTitleFromDatabase = (String) tPainting.selectAll().get(0)[1];
+
+        assertNotNull(afterArtistNameFromDatabase);
+        assertNotNull(afterPaintingTitleFromDatabase);
+
+        assertEquals((afterArtistNameFromDatabase).trim(), afterArtistNameFromContext);
+        assertEquals((afterPaintingTitleFromDatabase).trim(), afterPaintingTitleFromContext);
+    }
+
+    @Test
+    public void testObjectSelectQuery() throws Exception {
+        createTestDataWithDeletion();
+
+        ObjectSelect<CompoundPaintingLongNames> originalQuery = ObjectSelect.query(CompoundPaintingLongNames.class);
+
+        CompoundPaintingLongNames beforeCompoundPainting = originalQuery
+                .where(CompoundPaintingLongNames.PAINTING_ID_PK_PROPERTY.eq(1))
+                .selectOne(context);
+
+        String beforeArtistNameFromContext = beforeCompoundPainting.getArtistLongName();
+        String beforePaintingTitleFromContext = beforeCompoundPainting.getPaintingTitle();
+
+        String beforeArtistNameFromDatabase = (String) tArtist.selectAll().get(0)[1];
+        String beforePaintingTitleFromDatabase = (String) tPainting.selectAll().get(0)[1];
+
+        assertNotNull(beforeArtistNameFromDatabase);
+        assertNotNull(beforePaintingTitleFromDatabase);
+
+        assertEquals(beforeArtistNameFromDatabase.trim(), beforeArtistNameFromContext);
+        assertEquals(beforePaintingTitleFromDatabase.trim(), beforePaintingTitleFromContext);
+
+        beforeCompoundPainting.setArtistLongName("some");
+        beforeCompoundPainting.setPaintingTitle("omes");
+        context.commitChanges();
+
+        CompoundPaintingLongNames afterCompoundPainting = originalQuery
+                .where(CompoundPaintingLongNames.PAINTING_ID_PK_PROPERTY.eq(1))
+                .selectOne(context);
+
+        String afterArtistNameFromContext = afterCompoundPainting.getArtistLongName();
+        String afterPaintingTitleFromContext = afterCompoundPainting.getPaintingTitle();
+
+        String afterArtistNameFromDatabase = (String) tArtist.selectAll().get(0)[1];
+        String afterPaintingTitleFromDatabase = (String) tPainting.selectAll().get(0)[1];
+
+        assertNotNull(afterArtistNameFromDatabase);
+        assertNotNull(afterPaintingTitleFromDatabase);
+
+        assertEquals(afterArtistNameFromDatabase.trim(), afterArtistNameFromContext);
+        assertEquals(afterPaintingTitleFromDatabase.trim(), afterPaintingTitleFromContext);
+    }
+
     @Test
     public void testSelectEJQBQL() throws Exception {
         createTestDataSet();
@@ -377,4 +501,64 @@ public class DataContextFlattenedAttributesIT extends ServerCase {
 
         context.commitChanges();
     }
+
+    @Test
+    public void testUpdateDifferentContext() {
+        Object id;
+        {
+            // insert
+            ObjectContext context1 = runtime.newContext();
+            CompoundPainting o1 = context1.newObject(CompoundPainting.class);
+            o1.setArtistName("A1");
+            o1.setEstimatedPrice(new BigDecimal(1d));
+            o1.setGalleryName("G1");
+            o1.setPaintingTitle("P1");
+            o1.setTextReview("T1");
+
+            context1.commitChanges();
+            id = Cayenne.pkForObject(o1);
+        }
+
+        {
+            // read and update
+            ObjectContext context2 = runtime.newContext();
+            CompoundPainting o2 = SelectById.query(CompoundPainting.class, id).selectFirst(context2);
+
+            o2.setArtistName("AX1");
+            o2.setEstimatedPrice(new BigDecimal(2d));
+            o2.setGalleryName("XG1");
+            o2.setPaintingTitle("PX1");
+            o2.setTextReview("TX1");
+
+            context2.commitChanges();
+        }
+
+        {
+            // read and check
+            ObjectContext context3 = runtime.newContext();
+            CompoundPainting o3 = SelectById.query(CompoundPainting.class, id).selectFirst(context3);
+
+            assertEquals("AX1", o3.getArtistName());
+            assertEquals("2.00", o3.getEstimatedPrice().toPlainString());
+            assertEquals("XG1", o3.getGalleryName());
+            assertEquals("PX1", o3.getPaintingTitle());
+            assertEquals("TX1", o3.getTextReview());
+        }
+    }
+
+    /**
+     * Guarantee initial structure of database via deletion all data inside
+     */
+    private void createTestDataWithDeletion() throws Exception {
+        tPaintingInfo.deleteAll();
+        tPainting.deleteAll();
+        tGallery.deleteAll();
+        tArtist.deleteAll();
+
+        long dateBase = System.currentTimeMillis();
+        tArtist.insert(1, "artist1", new java.sql.Date(dateBase + 1000 * 60 * 60 * 24));
+        tGallery.insert(1, "gallery1");
+        tPainting.insert(1, "painting1", 1, new BigDecimal("1000"), 1);
+        tPaintingInfo.insert(1, "painting review1");
+    }
 }
