@@ -22,32 +22,28 @@ import org.apache.cayenne.Cayenne;
 import org.apache.cayenne.ObjectContext;
 import org.apache.cayenne.configuration.server.ServerRuntime;
 import org.apache.cayenne.di.Inject;
+import org.apache.cayenne.exp.property.PropertyFactory;
+import org.apache.cayenne.query.ColumnSelect;
 import org.apache.cayenne.query.EJBQLQuery;
 import org.apache.cayenne.query.ObjectSelect;
 import org.apache.cayenne.query.SelectById;
-import org.apache.cayenne.query.SelectQuery;
 import org.apache.cayenne.test.jdbc.DBHelper;
 import org.apache.cayenne.test.jdbc.TableHelper;
 import org.apache.cayenne.testdo.inheritance_vertical.*;
 import org.apache.cayenne.unit.di.server.CayenneProjects;
 import org.apache.cayenne.unit.di.server.ServerCase;
 import org.apache.cayenne.unit.di.server.UseServerRuntime;
-import org.junit.Before;
-import org.junit.Ignore;
 import org.junit.Test;
 
 import java.sql.SQLException;
 import java.sql.Types;
 import java.util.Collections;
+import java.util.Date;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertNotSame;
-import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.*;
 
 @UseServerRuntime(CayenneProjects.INHERITANCE_VERTICAL_PROJECT)
 public class VerticalInheritanceIT extends ServerCase {
@@ -61,43 +57,12 @@ public class VerticalInheritanceIT extends ServerCase {
 	@Inject
 	protected ServerRuntime runtime;
 
-	private TableHelper ivRootTable;
-	private TableHelper ivSub1Table;
-	private TableHelper ivSub2Table;
-	private TableHelper ivSub3Table;
-	private TableHelper ivSub1Sub1Table;
-	private TableHelper iv1RootTable;
-	private TableHelper iv1Sub1Table;
-
-
-	@Before
-	public void createTableHelpers() {
-		ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
-		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR").setColumnTypes(
-				Types.INTEGER, Types.VARCHAR, Types.VARCHAR);
-
-		ivSub1Table = new TableHelper(dbHelper, "IV_SUB1");
-		ivSub1Table.setColumns("ID", "SUB1_NAME");
-
-		ivSub2Table = new TableHelper(dbHelper, "IV_SUB2");
-		ivSub2Table.setColumns("ID", "SUB2_NAME", "SUB2_ATTR");
-
-		ivSub3Table = new TableHelper(dbHelper, "IV_SUB3");
-		ivSub3Table.setColumns("ID", "IV_ROOT_ID");
-
-		ivSub1Sub1Table = new TableHelper(dbHelper, "IV_SUB1_SUB1");
-		ivSub1Sub1Table.setColumns("ID", "SUB1_SUB1_NAME");
-
-		iv1RootTable = new TableHelper(dbHelper, "IV1_ROOT");
-		iv1RootTable.setColumns("ID", "NAME", "DISCRIMINATOR").setColumnTypes(
-				Types.INTEGER, Types.VARCHAR, Types.VARCHAR);
-
-		iv1Sub1Table = new TableHelper(dbHelper, "IV1_SUB1");
-		iv1Sub1Table.setColumns("ID", "SUB1_NAME");
-	}
-
     @Test
 	public void testInsert_Root() throws Exception {
+
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR");
+
 		assertEquals(0, ivRootTable.getRowCount());
 
 		IvRoot root = context.newObject(IvRoot.class);
@@ -116,12 +81,19 @@ public class VerticalInheritanceIT extends ServerCase {
 
     @Test
 	public void testInsert_Sub1() throws Exception {
+
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR");
+
+		TableHelper ivSub1Table = new TableHelper(dbHelper, "IV_SUB1");
+		ivSub1Table.setColumns("ID", "SUB1_NAME");
+
 		IvSub1 sub1 = context.newObject(IvSub1.class);
 		sub1.setName("XyZX");
 		sub1.getObjectContext().commitChanges();
 
 		assertEquals(1, ivRootTable.getRowCount());
-		assertEquals(1, ivSub1Table.getRowCount());
+		assertEquals(0, ivSub1Table.getRowCount());
 
 		Object[] data = ivRootTable.select();
 		assertEquals(3, data.length);
@@ -130,11 +102,6 @@ public class VerticalInheritanceIT extends ServerCase {
 		assertEquals("XyZX", data[1]);
 		assertEquals("IvSub1", data[2]);
 
-		Object[] subdata = ivSub1Table.select();
-		assertEquals(2, subdata.length);
-		assertEquals(data[0], subdata[0]);
-		assertNull(subdata[1]);
-
 		ivSub1Table.deleteAll();
 		ivRootTable.deleteAll();
 
@@ -150,7 +117,7 @@ public class VerticalInheritanceIT extends ServerCase {
 		assertEquals("XyZXY", data[1]);
 		assertEquals("IvSub1", data[2]);
 
-		subdata = ivSub1Table.select();
+		Object[] subdata = ivSub1Table.select();
 		assertEquals(2, subdata.length);
 		assertEquals(data[0], subdata[0]);
 		assertEquals("BdE2", subdata[1]);
@@ -158,12 +125,19 @@ public class VerticalInheritanceIT extends ServerCase {
 
     @Test
 	public void testInsert_Sub2() throws Exception {
+
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR");
+
+		TableHelper ivSub2Table = new TableHelper(dbHelper, "IV_SUB2");
+		ivSub2Table.setColumns("ID", "SUB2_NAME", "SUB2_ATTR");
+
 		IvSub2 sub2 = context.newObject(IvSub2.class);
 		sub2.setName("XyZX");
 		sub2.getObjectContext().commitChanges();
 
 		assertEquals(1, ivRootTable.getRowCount());
-		assertEquals(1, ivSub2Table.getRowCount());
+		assertEquals(0, ivSub2Table.getRowCount());
 
 		Object[] data = ivRootTable.select();
 		assertEquals(3, data.length);
@@ -172,12 +146,6 @@ public class VerticalInheritanceIT extends ServerCase {
 		assertEquals("XyZX", data[1]);
 		assertEquals("IvSub2", data[2]);
 
-		Object[] subdata = ivSub2Table.select();
-		assertEquals(3, subdata.length);
-		assertEquals(data[0], subdata[0]);
-		assertNull(subdata[1]);
-		assertNull(subdata[2]);
-
 		ivSub2Table.deleteAll();
 		ivRootTable.deleteAll();
 
@@ -194,7 +162,7 @@ public class VerticalInheritanceIT extends ServerCase {
 		assertEquals("XyZXY", data[1]);
 		assertEquals("IvSub2", data[2]);
 
-		subdata = ivSub2Table.select();
+		Object[] subdata = ivSub2Table.select();
 		assertEquals(3, subdata.length);
 		assertEquals(data[0], subdata[0]);
 		assertEquals("BdE2", subdata[1]);
@@ -221,6 +189,13 @@ public class VerticalInheritanceIT extends ServerCase {
 	 */
     @Test(expected = org.apache.cayenne.validation.ValidationException.class)
     public void testValidationOnInsert_Sub3_Exception() {
+
+        TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+        ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR");
+
+        TableHelper ivSub3Table = new TableHelper(dbHelper, "IV_SUB3");
+        ivSub3Table.setColumns("ID", "IV_ROOT_ID");
+
         IvSub3 sub3 = context.newObject(IvSub3.class);
         sub3.setName("XyZX");
 		context.commitChanges();
@@ -231,6 +206,13 @@ public class VerticalInheritanceIT extends ServerCase {
 	 */
 	@Test
 	public void testValidationOnInsert_Sub3_Ok() throws Exception {
+
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR");
+
+		TableHelper ivSub3Table = new TableHelper(dbHelper, "IV_SUB3");
+		ivSub3Table.setColumns("ID", "IV_ROOT_ID");
+
 		IvSub3 sub3 = context.newObject(IvSub3.class);
 		sub3.setName("XyZX");
 		sub3.setIvRoot(sub3);
@@ -245,10 +227,14 @@ public class VerticalInheritanceIT extends ServerCase {
 	 */
 	@Test
 	public void testUpdateRelation_Sub3() throws Exception {
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR");
 		ivRootTable.insert(1, "root1", null);
 		ivRootTable.insert(2, "root2", null);
 		ivRootTable.insert(3, "name", "IvSub3");
 
+		TableHelper ivSub3Table = new TableHelper(dbHelper, "IV_SUB3");
+		ivSub3Table.setColumns("ID", "IV_ROOT_ID");
 		ivSub3Table.insert(3, 1);
 
 		IvRoot root = SelectById.query(IvRoot.class, 2).selectOne(context);
@@ -274,9 +260,20 @@ public class VerticalInheritanceIT extends ServerCase {
 
     @Test
 	public void testInsert_Sub1Sub1() throws Exception {
+
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR");
+
+		TableHelper ivSub1Table = new TableHelper(dbHelper, "IV_SUB1");
+		ivSub1Table.setColumns("ID", "SUB1_NAME", "SUB1_PRICE");
+
+		TableHelper ivSub1Sub1Table = new TableHelper(dbHelper, "IV_SUB1_SUB1");
+		ivSub1Sub1Table.setColumns("ID", "SUB1_SUB1_NAME", "SUB1_SUB1_PRICE");
+
 		IvSub1Sub1 sub1Sub1 = context.newObject(IvSub1Sub1.class);
 		sub1Sub1.setName("XyZN");
 		sub1Sub1.setSub1Name("mDA");
+		sub1Sub1.setPrice(42.0);
 		sub1Sub1.setSub1Sub1Name("3DQa");
 		sub1Sub1.getObjectContext().commitChanges();
 
@@ -292,24 +289,34 @@ public class VerticalInheritanceIT extends ServerCase {
 		assertEquals("IvSub1Sub1", data[2]);
 
 		Object[] subdata = ivSub1Table.select();
-		assertEquals(2, subdata.length);
+		assertEquals(3, subdata.length);
 		assertEquals(data[0], subdata[0]);
 		assertEquals("mDA", subdata[1]);
+		assertNull(subdata[2]);
 
 		Object[] subsubdata = ivSub1Sub1Table.select();
-		assertEquals(2, subsubdata.length);
+		assertEquals(3, subsubdata.length);
 		assertEquals(data[0], subsubdata[0]);
 		assertEquals("3DQa", subsubdata[1]);
+		assertNull(subdata[2]);
 	}
 
     @Test
 	public void testSelectQuery_SuperSub() throws Exception {
+
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR").setColumnTypes(
+				Types.INTEGER, Types.VARCHAR, Types.VARCHAR);
+
+		TableHelper ivSub1Table = new TableHelper(dbHelper, "IV_SUB1");
+		ivSub1Table.setColumns("ID", "SUB1_NAME");
+
 		// insert
 		ivRootTable.insert(1, "xROOT", null);
 		ivRootTable.insert(2, "xSUB1_ROOT", "IvSub1");
 		ivSub1Table.insert(2, "xSUB1");
 
-		SelectQuery<IvRoot> query = new SelectQuery<>(IvRoot.class);
+		ObjectSelect<IvRoot> query = ObjectSelect.query(IvRoot.class);
 		List<IvRoot> results = context.select(query);
 
 		assertEquals(2, results.size());
@@ -338,6 +345,20 @@ public class VerticalInheritanceIT extends ServerCase {
 
     @Test
 	public void testSelectQuery_DeepAndWide() throws Exception {
+
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR").setColumnTypes(
+				Types.INTEGER, Types.VARCHAR, Types.VARCHAR);
+
+		TableHelper ivSub1Table = new TableHelper(dbHelper, "IV_SUB1");
+		ivSub1Table.setColumns("ID", "SUB1_NAME");
+
+		TableHelper ivSub2Table = new TableHelper(dbHelper, "IV_SUB2");
+		ivSub2Table.setColumns("ID", "SUB2_NAME");
+
+		TableHelper ivSub1Sub1Table = new TableHelper(dbHelper, "IV_SUB1_SUB1");
+		ivSub1Sub1Table.setColumns("ID", "SUB1_SUB1_NAME");
+
 		// insert
 		ivRootTable.insert(1, "xROOT", null);
 
@@ -349,9 +370,9 @@ public class VerticalInheritanceIT extends ServerCase {
 		ivSub1Sub1Table.insert(3, "xSUB1_SUB1");
 
 		ivRootTable.insert(4, "xROOT_SUB2", "IvSub2");
-		ivSub2Table.insert(4, "xSUB2", "");
+		ivSub2Table.insert(4, "xSUB2");
 
-		SelectQuery<IvRoot> query = new SelectQuery<>(IvRoot.class);
+		ObjectSelect<IvRoot> query = ObjectSelect.query(IvRoot.class);
 		List<IvRoot> results = context.select(query);
 
 		assertEquals(4, results.size());
@@ -394,6 +415,20 @@ public class VerticalInheritanceIT extends ServerCase {
 
     @Test
 	public void testSelectQuery_MiddleLeaf() throws Exception {
+
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR").setColumnTypes(
+				Types.INTEGER, Types.VARCHAR, Types.VARCHAR);
+
+		TableHelper ivSub1Table = new TableHelper(dbHelper, "IV_SUB1");
+		ivSub1Table.setColumns("ID", "SUB1_NAME");
+
+		TableHelper ivSub2Table = new TableHelper(dbHelper, "IV_SUB2");
+		ivSub2Table.setColumns("ID", "SUB2_NAME");
+
+		TableHelper ivSub1Sub1Table = new TableHelper(dbHelper, "IV_SUB1_SUB1");
+		ivSub1Sub1Table.setColumns("ID", "SUB1_SUB1_NAME");
+
 		// insert
 		ivRootTable.insert(1, "xROOT", null);
 
@@ -405,9 +440,9 @@ public class VerticalInheritanceIT extends ServerCase {
 		ivSub1Sub1Table.insert(3, "xSUB1_SUB1");
 
 		ivRootTable.insert(4, "xROOT_SUB2", "IvSub2");
-		ivSub2Table.insert(4, "xSUB2", "");
+		ivSub2Table.insert(4, "xSUB2");
 
-		SelectQuery<IvSub1> query = new SelectQuery<>(IvSub1.class);
+		ObjectSelect<IvSub1> query = ObjectSelect.query(IvSub1.class);
 		List<IvSub1> results = context.select(query);
 
 		assertEquals(2, results.size());
@@ -439,6 +474,20 @@ public class VerticalInheritanceIT extends ServerCase {
 
     @Test
 	public void testDelete_Mix() throws Exception {
+
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR").setColumnTypes(
+				Types.INTEGER, Types.VARCHAR, Types.VARCHAR);
+
+		TableHelper ivSub1Table = new TableHelper(dbHelper, "IV_SUB1");
+		ivSub1Table.setColumns("ID", "SUB1_NAME");
+
+		TableHelper ivSub2Table = new TableHelper(dbHelper, "IV_SUB2");
+		ivSub2Table.setColumns("ID", "SUB2_NAME");
+
+		TableHelper ivSub1Sub1Table = new TableHelper(dbHelper, "IV_SUB1_SUB1");
+		ivSub1Sub1Table.setColumns("ID", "SUB1_SUB1_NAME");
+
 		// insert
 		ivRootTable.insert(1, "xROOT", null);
 
@@ -450,11 +499,11 @@ public class VerticalInheritanceIT extends ServerCase {
 		ivSub1Sub1Table.insert(3, "xSUB1_SUB1");
 
 		ivRootTable.insert(4, "xROOT_SUB2", "IvSub2");
-		ivSub2Table.insert(4, "xSUB2", "");
+		ivSub2Table.insert(4, "xSUB2");
 
-		SelectQuery<IvRoot> query = new SelectQuery<>(IvRoot.class);
+		ObjectSelect<IvRoot> query = ObjectSelect.query(IvRoot.class);
 
-		List<IvRoot> results = context.select(query);
+		List<IvRoot> results = query.select(context);
 
 		assertEquals(4, results.size());
 		Map<String, IvRoot> resultTypes = new HashMap<>();
@@ -483,13 +532,22 @@ public class VerticalInheritanceIT extends ServerCase {
 	}
 
     @Test
+	@Deprecated
 	public void testSelectQuery_AttributeOverrides() throws Exception {
+
+		TableHelper iv1RootTable = new TableHelper(dbHelper, "IV1_ROOT");
+		iv1RootTable.setColumns("ID", "NAME", "DISCRIMINATOR").setColumnTypes(
+				Types.INTEGER, Types.VARCHAR, Types.VARCHAR);
+
+		TableHelper iv1Sub1Table = new TableHelper(dbHelper, "IV1_SUB1");
+		iv1Sub1Table.setColumns("ID", "SUB1_NAME");
+
 		// insert
 		iv1RootTable.insert(1, "xROOT", null);
 		iv1RootTable.insert(2, "xSUB1_ROOT", "Iv1Sub1");
 		iv1Sub1Table.insert(2, "xSUB1");
 
-		SelectQuery<Iv1Root> query = new SelectQuery<>(Iv1Root.class);
+		ObjectSelect<Iv1Root> query = ObjectSelect.query(Iv1Root.class);
 		List<Iv1Root> results = context.select(query);
 
 		assertEquals(2, results.size());
@@ -584,6 +642,7 @@ public class VerticalInheritanceIT extends ServerCase {
 
 		IvImpl impl = context.newObject(IvImpl.class);
 		impl.setName("Impl 1");
+		impl.setAttr0(new Date());
 		impl.setAttr1("attr1");
 		impl.setAttr2("attr2");
 		impl.setOther1(other1);
@@ -658,7 +717,17 @@ public class VerticalInheritanceIT extends ServerCase {
 
 	@Test
 	public void testCountEjbqlQuery() throws Exception {
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR");
+
+		TableHelper ivSub1Table = new TableHelper(dbHelper, "IV_SUB1");
+		ivSub1Table.setColumns("ID", "SUB1_NAME");
+
+		TableHelper ivSub2Table = new TableHelper(dbHelper, "IV_SUB2");
+		ivSub2Table.setColumns("ID", "SUB2_ATTR", "SUB2_NAME");
+
 		// Root, IvSub1, IvSub2
+
 		ivRootTable.insert(1, "root1", "");
 
 		ivRootTable.insert(2, "sub11", "IvSub1");
@@ -666,8 +735,8 @@ public class VerticalInheritanceIT extends ServerCase {
 
 		ivRootTable.insert(3, "sub21", "IvSub2");
 		ivRootTable.insert(4, "sub22", "IvSub2");
-		ivSub2Table.insert(3, "sub_name2_1", "attr1");
-		ivSub2Table.insert(4, "sub_name2_2", "attr2");
+		ivSub2Table.insert(3, "attr1", "sub_name2_1");
+		ivSub2Table.insert(4, "attr2", "sub_name2_2");
 
 		EJBQLQuery query1 = new EJBQLQuery("SELECT COUNT(a) FROM IvRoot a");
 		assertEquals(Collections.singletonList(4L), context.performQuery(query1));
@@ -680,7 +749,6 @@ public class VerticalInheritanceIT extends ServerCase {
 	}
 
 	@Test
-	@Ignore("CAY-2682. Fixed in 4.2 as the FlushAction is reimplemented")
 	public void testPropagatedGeneratedPK() {
 		IvGenKeySub sub = context.newObject(IvGenKeySub.class);
 		sub.setName("test");
@@ -688,4 +756,101 @@ public class VerticalInheritanceIT extends ServerCase {
 
 		assertTrue(Cayenne.intPKForObject(sub) > 0);
 	}
+
+	@Test
+	public void testColumnSelectVerticalInheritance_Sub1() throws SQLException {
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR");
+
+		TableHelper ivSub1Table = new TableHelper(dbHelper, "IV_SUB1");
+		ivSub1Table.setColumns("ID", "SUB1_NAME", "SUB1_PRICE");
+
+		TableHelper ivSub1Sub1Table = new TableHelper(dbHelper, "IV_SUB1_SUB1");
+		ivSub1Sub1Table.setColumns("ID", "SUB1_SUB1_NAME", "SUB1_SUB1_PRICE");
+
+		IvSub1Sub1 sub1Sub1 = context.newObject(IvSub1Sub1.class);
+		sub1Sub1.setName("XyZN");
+		sub1Sub1.setSub1Name("mDA");
+		sub1Sub1.setPrice(42.0);
+		sub1Sub1.setSub1Sub1Name("3DQa");
+		sub1Sub1.getObjectContext().commitChanges();
+
+		assertEquals(1, ivRootTable.getRowCount());
+		assertEquals(1, ivSub1Table.getRowCount());
+		assertEquals(1, ivSub1Sub1Table.getRowCount());
+
+		Object[] data = ivRootTable.select();
+		assertEquals(3, data.length);
+		assertTrue(data[0] instanceof Number);
+		assertTrue(((Number) data[0]).intValue() > 0);
+		assertEquals("XyZN", data[1]);
+		assertEquals("IvSub1Sub1", data[2]);
+
+		Object[] subdata = ivSub1Table.select();
+		assertEquals(3, subdata.length);
+		assertEquals(data[0], subdata[0]);
+		assertEquals("mDA", subdata[1]);
+
+		Object[] subsubdata = ivSub1Sub1Table.select();
+		assertEquals(3, subsubdata.length);
+		assertEquals(data[0], subsubdata[0]);
+		assertEquals("3DQa", subsubdata[1]);
+
+		ColumnSelect<IvSub1> originalQueryForSub1 = ObjectSelect.query(IvSub1.class)
+				.column(PropertyFactory.createSelf(IvSub1.class));
+
+		IvSub1 result = originalQueryForSub1.selectOne(context);
+		assertEquals("XyZN", result.getName());
+		assertEquals(Double.valueOf(42.0), result.getPrice());
+		assertEquals("mDA", result.getSub1Name());
+	}
+
+	@Test
+	public void testColumnSelectVerticalInheritance_Sub1Sub1() throws SQLException {
+		TableHelper ivRootTable = new TableHelper(dbHelper, "IV_ROOT");
+		ivRootTable.setColumns("ID", "NAME", "DISCRIMINATOR");
+
+		TableHelper ivSub1Table = new TableHelper(dbHelper, "IV_SUB1");
+		ivSub1Table.setColumns("ID", "SUB1_NAME", "SUB1_PRICE");
+
+		TableHelper ivSub1Sub1Table = new TableHelper(dbHelper, "IV_SUB1_SUB1");
+		ivSub1Sub1Table.setColumns("ID", "SUB1_SUB1_NAME", "SUB1_SUB1_PRICE");
+
+		IvSub1Sub1 sub1Sub1 = context.newObject(IvSub1Sub1.class);
+		sub1Sub1.setName("XyZN");
+		sub1Sub1.setSub1Name("mDA");
+		sub1Sub1.setPrice(42.0);
+		sub1Sub1.setSub1Sub1Name("3DQa");
+		sub1Sub1.getObjectContext().commitChanges();
+
+		assertEquals(1, ivRootTable.getRowCount());
+		assertEquals(1, ivSub1Table.getRowCount());
+		assertEquals(1, ivSub1Sub1Table.getRowCount());
+
+		Object[] data = ivRootTable.select();
+		assertEquals(3, data.length);
+		assertTrue(data[0] instanceof Number);
+		assertTrue(((Number) data[0]).intValue() > 0);
+		assertEquals("XyZN", data[1]);
+		assertEquals("IvSub1Sub1", data[2]);
+
+		Object[] subdata = ivSub1Table.select();
+		assertEquals(3, subdata.length);
+		assertEquals(data[0], subdata[0]);
+		assertEquals("mDA", subdata[1]);
+
+		Object[] subsubdata = ivSub1Sub1Table.select();
+		assertEquals(3, subsubdata.length);
+		assertEquals(data[0], subsubdata[0]);
+		assertEquals("3DQa", subsubdata[1]);
+
+		ColumnSelect<IvSub1Sub1> originalQueryForSub1Sub1 = ObjectSelect.query(IvSub1Sub1.class)
+				.column(PropertyFactory.createSelf(IvSub1Sub1.class));
+
+		IvSub1Sub1 result = originalQueryForSub1Sub1.selectOne(context);
+		assertEquals("XyZN", result.getName());
+		assertEquals(Double.valueOf(42.0), result.getPrice());
+		assertEquals("mDA", result.getSub1Name());
+		assertEquals("3DQa", result.getSub1Sub1Name());
+	}
 }
