@@ -33,17 +33,19 @@ import org.apache.cayenne.Fault;
 import org.apache.cayenne.ObjectContext;
 import org.apache.cayenne.PersistenceState;
 import org.apache.cayenne.ResultBatchIterator;
-import org.apache.cayenne.ResultIteratorCallback;
 import org.apache.cayenne.access.DataContext;
 import org.apache.cayenne.configuration.server.ServerRuntime;
 import org.apache.cayenne.di.Inject;
-import org.apache.cayenne.exp.Expression;
 import org.apache.cayenne.exp.ExpressionFactory;
-import org.apache.cayenne.exp.FunctionExpressionFactory;
-import org.apache.cayenne.exp.Property;
+import org.apache.cayenne.exp.property.EntityProperty;
+import org.apache.cayenne.exp.property.NumericProperty;
+import org.apache.cayenne.exp.property.PropertyFactory;
+import org.apache.cayenne.exp.property.StringProperty;
 import org.apache.cayenne.test.jdbc.DBHelper;
 import org.apache.cayenne.test.jdbc.TableHelper;
 import org.apache.cayenne.testdo.testmap.Artist;
+import org.apache.cayenne.testdo.testmap.ArtistExhibit;
+import org.apache.cayenne.testdo.testmap.Exhibit;
 import org.apache.cayenne.testdo.testmap.Gallery;
 import org.apache.cayenne.testdo.testmap.Painting;
 import org.apache.cayenne.testdo.testmap.PaintingInfo;
@@ -56,6 +58,7 @@ import org.junit.Before;
 import org.junit.Ignore;
 import org.junit.Test;
 
+import static org.hamcrest.CoreMatchers.instanceOf;
 import static org.junit.Assert.*;
 
 /**
@@ -110,7 +113,7 @@ public class ColumnSelectIT extends ServerCase {
     @Test
     public void testSelectGroupBy() throws Exception {
         Object[] result = ObjectSelect.query(Artist.class)
-                .columns(Artist.DATE_OF_BIRTH, Property.COUNT)
+                .columns(Artist.DATE_OF_BIRTH, PropertyFactory.COUNT)
                 .orderBy(Artist.DATE_OF_BIRTH.asc())
                 .selectFirst(context);
 
@@ -121,8 +124,7 @@ public class ColumnSelectIT extends ServerCase {
     @Test
     public void testSelectSimpleHaving() throws Exception {
         Object[] result = ObjectSelect.query(Artist.class)
-                .columns(Artist.DATE_OF_BIRTH, Property.COUNT)
-                .orderBy(Artist.DATE_OF_BIRTH.asc())
+                .columns(Artist.DATE_OF_BIRTH, PropertyFactory.COUNT)
                 .having(Artist.DATE_OF_BIRTH.eq(dateFormat.parse("1/2/17")))
                 .selectOne(context);
 
@@ -131,10 +133,10 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test(expected = Exception.class)
-    public void testHavingOnNonGroupByColumn() throws Exception {
-        Property<String> nameSubstr = Artist.ARTIST_NAME.substring(1, 6);
+    public void testHavingOnNonGroupByColumn() {
+        StringProperty<String> nameSubstr = Artist.ARTIST_NAME.substring(1, 6);
 
-        Object[] q = ObjectSelect.columnQuery(Artist.class, nameSubstr, Property.COUNT)
+        Object[] q = ObjectSelect.columnQuery(Artist.class, nameSubstr, PropertyFactory.COUNT)
                 .having(Artist.ARTIST_NAME.like("artist%"))
                 .selectOne(context);
         assertEquals("artist", q[0]);
@@ -152,13 +154,13 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testSelectHavingWithExpressionAlias() throws Exception {
+    public void testSelectHavingWithExpressionAlias() {
 
         Object[] q = null;
         try {
             q = ObjectSelect
-                    .columnQuery(Artist.class, Artist.ARTIST_NAME.substring(1, 6).alias("name_substr"), Property.COUNT)
-                    .having(Property.COUNT.gt(10L))
+                    .columnQuery(Artist.class, Artist.ARTIST_NAME.substring(1, 6).alias("name_substr"), PropertyFactory.COUNT)
+                    .having(PropertyFactory.COUNT.gt(10L))
                     .selectOne(context);
         } catch (CayenneRuntimeException ex) {
             if(unitDbAdapter.supportsExpressionInHaving()) {
@@ -173,12 +175,12 @@ public class ColumnSelectIT extends ServerCase {
 
     @Ignore("Need to figure out a better way to handle alias / no alias case for expression in having")
     @Test
-    public void testSelectHavingWithExpressionNoAlias() throws Exception {
+    public void testSelectHavingWithExpressionNoAlias() {
 
         Object[] q = null;
         try {
-            q = ObjectSelect.columnQuery(Artist.class, Artist.ARTIST_NAME.substring(1, 6), Property.COUNT)
-                    .having(Property.COUNT.gt(10L))
+            q = ObjectSelect.columnQuery(Artist.class, Artist.ARTIST_NAME.substring(1, 6), PropertyFactory.COUNT)
+                    .having(PropertyFactory.COUNT.gt(10L))
                     .selectOne(context);
         } catch (CayenneRuntimeException ex) {
             if(unitDbAdapter.supportsExpressionInHaving()) {
@@ -192,12 +194,12 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testSelectWhereAndHaving() throws Exception {
+    public void testSelectWhereAndHaving() {
         Object[] q = null;
         try {
-            q = ObjectSelect.columnQuery(Artist.class, Artist.ARTIST_NAME.substring(1, 6).alias("name_substr"), Property.COUNT)
+            q = ObjectSelect.columnQuery(Artist.class, Artist.ARTIST_NAME.substring(1, 6).alias("name_substr"), PropertyFactory.COUNT)
                     .where(Artist.ARTIST_NAME.substring(1, 1).eq("a"))
-                    .having(Property.COUNT.gt(10L))
+                    .having(PropertyFactory.COUNT.gt(10L))
                     .selectOne(context);
         } catch (CayenneRuntimeException ex) {
             if(unitDbAdapter.supportsExpressionInHaving()) {
@@ -211,7 +213,7 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testHavingWithoutAggregate() throws Exception {
+    public void testHavingWithoutAggregate() {
         Object date = ObjectSelect.columnQuery(Artist.class, Artist.DATE_OF_BIRTH, Artist.ARTIST_NAME)
                 .having(Artist.ARTIST_NAME.like("a%"))
                 .selectFirst(context);
@@ -228,7 +230,7 @@ public class ColumnSelectIT extends ServerCase {
      */
     @Ignore
     @Test
-    public void testHavingWithoutSelect() throws Exception {
+    public void testHavingWithoutSelect() {
         Object date = ObjectSelect.columnQuery(Artist.class, Artist.DATE_OF_BIRTH)
                 .having(Artist.ARTIST_NAME.like("a%"))
                 .selectFirst(context);
@@ -241,7 +243,7 @@ public class ColumnSelectIT extends ServerCase {
      *      SELECT a.name FROM artist a JOIN painting p ON (..) HAVING COUNT(p.id) > 4
      */
     @Test
-    public void testSelectRelationshipCountHavingWithoutFieldSelect() throws Exception {
+    public void testSelectRelationshipCountHavingWithoutFieldSelect() {
         Object[] result = null;
         try {
             result = ObjectSelect.query(Artist.class)
@@ -260,8 +262,8 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testSelectRelationshipCountHaving() throws Exception {
-        Property<Long> paintingCount = Artist.PAINTING_ARRAY.count();
+    public void testSelectRelationshipCountHaving() {
+        NumericProperty<Long> paintingCount = Artist.PAINTING_ARRAY.count();
 
         Object[] result = null;
         try {
@@ -281,13 +283,13 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testSelectWithQuoting() throws Exception {
+    public void testSelectWithQuoting() {
         if(unitDbAdapter instanceof PostgresUnitDbAdapter) {
             // we need to convert somehow all names to lowercase on postgres, so skip it for now
             return;
         }
 
-        Property<Long> paintingCount = Artist.PAINTING_ARRAY.count();
+        NumericProperty<Long> paintingCount = Artist.PAINTING_ARRAY.count();
         context.getEntityResolver().getDataMap("testmap").setQuotingSQLIdentifiers(true);
 
         Object[] result = null;
@@ -319,7 +321,7 @@ public class ColumnSelectIT extends ServerCase {
         context.getEntityResolver().getDataMap("testmap").setQuotingSQLIdentifiers(true);
         try {
             Object[] result = ObjectSelect.query(Artist.class)
-                    .columns(Artist.DATE_OF_BIRTH, Property.COUNT)
+                    .columns(Artist.DATE_OF_BIRTH, PropertyFactory.COUNT)
                     .orderBy(Artist.DATE_OF_BIRTH.asc())
                     .selectFirst(context);
 
@@ -331,16 +333,17 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testAgregateOnRelation() throws Exception {
+    public void testAgregateOnRelation() {
         BigDecimal min = new BigDecimal(3);
         BigDecimal max = new BigDecimal(30);
         BigDecimal avg = new BigDecimal(BigInteger.valueOf(1290L), 2);
         BigDecimal sum = new BigDecimal(258);
 
-        Property<BigDecimal> estimatedPrice = Artist.PAINTING_ARRAY.dot(Painting.ESTIMATED_PRICE);
+        NumericProperty<BigDecimal> estimatedPrice = Artist.PAINTING_ARRAY.dot(Painting.ESTIMATED_PRICE);
         Object[] minMaxAvgPrice = ObjectSelect.query(Artist.class)
                 .where(estimatedPrice.gte(min))
-                .min(estimatedPrice).max(estimatedPrice)
+                .min(estimatedPrice)
+                .max(estimatedPrice)
                 .avg(estimatedPrice)
                 .sum(estimatedPrice)
                 .count()
@@ -354,9 +357,9 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testQueryCount() throws Exception {
+    public void testQueryCount() {
         long count = ObjectSelect
-                .columnQuery(Artist.class, Property.COUNT)
+                .columnQuery(Artist.class, PropertyFactory.COUNT)
                 .selectOne(context);
 
         assertEquals(20, count);
@@ -381,7 +384,7 @@ public class ColumnSelectIT extends ServerCase {
         tArtist.insert(22, "artist_21", null);
 
         long count = ObjectSelect
-                .columnQuery(Artist.class, Property.COUNT)
+                .columnQuery(Artist.class, PropertyFactory.COUNT)
                 .selectOne(context);
         assertEquals(22, count);
 
@@ -399,13 +402,13 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testSelectFirst_MultiColumns() throws Exception {
+    public void testSelectFirst_MultiColumns() {
         Object[] a = ObjectSelect.query(Artist.class)
                 .columns(Artist.ARTIST_NAME, Artist.DATE_OF_BIRTH)
                 .columns(Artist.ARTIST_NAME, Artist.DATE_OF_BIRTH)
                 .columns(Artist.ARTIST_NAME.alias("newName"))
                 .where(Artist.ARTIST_NAME.like("artist%"))
-                .orderBy("db:ARTIST_ID")
+                .orderBy(Artist.ARTIST_ID_PK_PROPERTY.asc())
                 .selectFirst(context);
         assertNotNull(a);
         assertEquals("artist1", a[0]);
@@ -413,20 +416,19 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testSelectFirst_SingleValueInColumns() throws Exception {
+    public void testSelectFirst_SingleValueInColumns() {
         Object[] a = ObjectSelect.query(Artist.class)
                 .columns(Artist.ARTIST_NAME)
                 .where(Artist.ARTIST_NAME.like("artist%"))
-                .orderBy("db:ARTIST_ID")
+                .orderBy(Artist.ARTIST_ID_PK_PROPERTY.asc())
                 .selectFirst(context);
         assertNotNull(a);
         assertEquals("artist1", a[0]);
     }
 
     @Test
-    public void testSelectFirst_SubstringName() throws Exception {
-        Expression exp = FunctionExpressionFactory.substringExp(Artist.ARTIST_NAME.path(), 5, 3);
-        Property<String> substrName = Property.create("substrName", exp, String.class);
+    public void testSelectFirst_SubstringName() {
+        StringProperty<String> substrName = Artist.ARTIST_NAME.substring(5, 3);
         Object[] a = ObjectSelect.query(Artist.class)
                 .columns(Artist.ARTIST_NAME, substrName)
                 .where(substrName.eq("st3"))
@@ -438,9 +440,9 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testSelectFirst_RelColumns() throws Exception {
+    public void testSelectFirst_RelColumns() {
         // set shorter than painting_array.paintingTitle alias as some DBs doesn't support dot in alias
-        Property<String> paintingTitle = Artist.PAINTING_ARRAY.dot(Painting.PAINTING_TITLE).alias("paintingTitle");
+        StringProperty<String> paintingTitle = Artist.PAINTING_ARRAY.dot(Painting.PAINTING_TITLE).alias("paintingTitle");
 
         Object[] a = ObjectSelect.query(Artist.class)
                 .columns(Artist.ARTIST_NAME, paintingTitle)
@@ -451,9 +453,9 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testSelectFirst_RelColumn() throws Exception {
+    public void testSelectFirst_RelColumn() {
         // set shorter than painting_array.paintingTitle alias as some DBs doesn't support dot in alias
-        Property<String> paintingTitle = Artist.PAINTING_ARRAY.dot(Painting.PAINTING_TITLE).alias("paintingTitle");
+        StringProperty<String> paintingTitle = Artist.PAINTING_ARRAY.dot(Painting.PAINTING_TITLE).alias("paintingTitle");
 
         String a = ObjectSelect.query(Artist.class)
                 .column(paintingTitle)
@@ -464,8 +466,8 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testSelectFirst_RelColumnWithFunction() throws Exception {
-        Property<String> altTitle = Artist.PAINTING_ARRAY.dot(Painting.PAINTING_TITLE)
+    public void testSelectFirst_RelColumnWithFunction() {
+        StringProperty<String> altTitle = Artist.PAINTING_ARRAY.dot(Painting.PAINTING_TITLE)
                 .substring(7, 3).concat(" ", Artist.ARTIST_NAME)
                 .alias("altTitle");
 
@@ -484,14 +486,15 @@ public class ColumnSelectIT extends ServerCase {
         // test that all table aliases are correct
         List<Object[]> result = ObjectSelect.columnQuery(Artist.class,
                 Artist.PAINTING_ARRAY.outer().count(),
-                Property.createSelf(Artist.class),
+                PropertyFactory.createSelf(Artist.class),
                 Artist.PAINTING_ARRAY.dot(Painting.PAINTING_TITLE),
-                Property.createSelf(Artist.class),
+                PropertyFactory.createSelf(Artist.class),
                 Artist.PAINTING_ARRAY.dot(Painting.TO_GALLERY).dot(Gallery.GALLERY_NAME),
                 Artist.ARTIST_NAME,
-                Property.createSelf(Artist.class)
+                PropertyFactory.createSelf(Artist.class)
         ).select(context);
         assertEquals(21, result.size());
+
         for(Object[] next : result) {
             long count = (Long)next[0];
             Artist artist = (Artist)next[1];
@@ -502,11 +505,12 @@ public class ColumnSelectIT extends ServerCase {
             Artist artist3 = (Artist)next[6];
 
             assertTrue(paintingTitle.startsWith("painting"));
-            assertTrue(count == 4L || count == 5L);
             assertEquals("tate modern", galleryName);
             assertEquals(PersistenceState.COMMITTED, artist.getPersistenceState());
             assertEquals(PersistenceState.COMMITTED, artist2.getPersistenceState());
             assertEquals(PersistenceState.COMMITTED, artist3.getPersistenceState());
+            assertEquals(artistName, artist.getArtistName());
+            assertTrue(count == 4L || count == 5L);
         }
     }
 
@@ -515,23 +519,20 @@ public class ColumnSelectIT extends ServerCase {
      */
 
     @Test
-    public void testIterationSingleColumn() throws Exception {
+    public void testIterationSingleColumn() {
         ColumnSelect<String> columnSelect = ObjectSelect.query(Artist.class).column(Artist.ARTIST_NAME);
 
         final int[] count = new int[1];
-        columnSelect.iterate(context, new ResultIteratorCallback<String>() {
-            @Override
-            public void next(String object) {
-                count[0]++;
-                assertTrue(object.startsWith("artist"));
-            }
+        columnSelect.iterate(context, object -> {
+            count[0]++;
+            assertTrue(object.startsWith("artist"));
         });
 
         assertEquals(20, count[0]);
     }
 
     @Test
-    public void testBatchIterationSingleColumn() throws Exception {
+    public void testBatchIterationSingleColumn() {
         ColumnSelect<String> columnSelect = ObjectSelect.query(Artist.class).column(Artist.ARTIST_NAME);
 
         try(ResultBatchIterator<String> it = columnSelect.batchIterator(context, 10)) {
@@ -542,24 +543,21 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testIterationMultiColumns() throws Exception {
+    public void testIterationMultiColumns() {
         ColumnSelect<Object[]> columnSelect = ObjectSelect.query(Artist.class).columns(Artist.ARTIST_NAME, Artist.DATE_OF_BIRTH);
 
         final int[] count = new int[1];
-        columnSelect.iterate(context, new ResultIteratorCallback<Object[]>() {
-            @Override
-            public void next(Object[] object) {
-                count[0]++;
-                assertTrue(object[0] instanceof String);
-                assertTrue(object[1] instanceof java.util.Date);
-            }
+        columnSelect.iterate(context, object -> {
+            count[0]++;
+            assertTrue(object[0] instanceof String);
+            assertTrue(object[1] instanceof Date);
         });
 
         assertEquals(20, count[0]);
     }
 
     @Test
-    public void testBatchIterationMultiColumns() throws Exception {
+    public void testBatchIterationMultiColumns() {
         ColumnSelect<Object[]> columnSelect = ObjectSelect.query(Artist.class).columns(Artist.ARTIST_NAME, Artist.DATE_OF_BIRTH);
 
         try(ResultBatchIterator<Object[]> it = columnSelect.batchIterator(context, 10)) {
@@ -625,7 +623,7 @@ public class ColumnSelectIT extends ServerCase {
 
     @Test
     public void testPageSizeOneObject() {
-        Property<Artist> artistFull = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistFull = PropertyFactory.createSelf(Artist.class);
         List<Artist> a = ObjectSelect.query(Artist.class)
                 .column(artistFull)
                 .pageSize(10)
@@ -639,7 +637,7 @@ public class ColumnSelectIT extends ServerCase {
 
     @Test
     public void testPageSizeOneObjectAsArray() {
-        Property<Artist> artistFull = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistFull = PropertyFactory.createSelf(Artist.class);
         List<Object[]> a = ObjectSelect.query(Artist.class)
                 .columns(artistFull)
                 .pageSize(10)
@@ -654,7 +652,7 @@ public class ColumnSelectIT extends ServerCase {
 
     @Test
     public void testPageSizeObjectAndScalars() {
-        Property<Artist> artistFull = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistFull = PropertyFactory.createSelf(Artist.class);
         List<Object[]> a = ObjectSelect.query(Artist.class)
                 .columns(Artist.ARTIST_NAME, artistFull, Artist.PAINTING_ARRAY.count())
                 .pageSize(10)
@@ -673,11 +671,12 @@ public class ColumnSelectIT extends ServerCase {
 
     @Test
     public void testPageSizeObjects() {
-        Property<Artist> artistFull = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistFull = PropertyFactory.createSelf(Artist.class);
         List<Object[]> a = ObjectSelect.query(Artist.class)
-                .columns(Artist.ARTIST_NAME, artistFull, Artist.PAINTING_ARRAY.flat(Painting.class))
+                .columns(Artist.ARTIST_NAME, artistFull, Artist.PAINTING_ARRAY.flat())
                 .pageSize(10)
                 .select(context);
+
         assertNotNull(a);
         assertEquals(21, a.size());
         int idx = 0;
@@ -685,7 +684,9 @@ public class ColumnSelectIT extends ServerCase {
             assertNotNull(next);
             assertEquals("" + idx, String.class, next[0].getClass());
             assertEquals("" + idx, Artist.class, next[1].getClass());
-            assertEquals("" + idx, Painting.class, next[2].getClass());
+            if(next[2] != null) {
+                assertEquals("" + idx, Painting.class, next[2].getClass());
+            }
             idx++;
         }
     }
@@ -696,7 +697,7 @@ public class ColumnSelectIT extends ServerCase {
 
     @Test
     public void testObjectColumnWithJointPrefetch() {
-        Property<Artist> artistFull = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistFull = PropertyFactory.createSelf(Artist.class);
 
         List<Object[]> result = ObjectSelect.query(Artist.class)
                 .columns(artistFull, Artist.DATE_OF_BIRTH, Artist.PAINTING_ARRAY.dot(Painting.PAINTING_TITLE))
@@ -708,7 +709,7 @@ public class ColumnSelectIT extends ServerCase {
 
     @Test
     public void testObjectColumnWithDisjointPrefetch() {
-        Property<Artist> artistFull = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistFull = PropertyFactory.createSelf(Artist.class);
 
         List<Object[]> result = ObjectSelect.query(Artist.class)
                 .columns(artistFull, Artist.DATE_OF_BIRTH, Artist.PAINTING_ARRAY.dot(Painting.PAINTING_TITLE))
@@ -720,7 +721,7 @@ public class ColumnSelectIT extends ServerCase {
 
     @Test
     public void testObjectColumnWithDisjointByIdPrefetch() {
-        Property<Artist> artistFull = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistFull = PropertyFactory.createSelf(Artist.class);
 
         List<Object[]> result = ObjectSelect.query(Artist.class)
                 .columns(artistFull, Artist.DATE_OF_BIRTH, Artist.PAINTING_ARRAY.dot(Painting.PAINTING_TITLE))
@@ -733,9 +734,9 @@ public class ColumnSelectIT extends ServerCase {
     private void checkPrefetchResults(List<Object[]> result) {
         assertEquals(21, result.size());
         for(Object[] next : result) {
-            assertTrue(next[0] instanceof Artist);
-            assertTrue(next[1] instanceof java.util.Date);
-            assertTrue(next[2] instanceof String);
+            assertThat(next[0], instanceOf(Artist.class));
+            assertThat(next[1], instanceOf(java.util.Date.class));
+            assertThat(next[2], instanceOf(String.class));
             Artist artist = (Artist)next[0];
             assertEquals(PersistenceState.COMMITTED, artist.getPersistenceState());
 
@@ -747,7 +748,7 @@ public class ColumnSelectIT extends ServerCase {
 
     @Test
     public void testAggregateColumnWithJointPrefetch() {
-        Property<Artist> artistFull = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistFull = PropertyFactory.createSelf(Artist.class);
 
         List<Object[]> result = ObjectSelect.query(Artist.class)
                 .columns(artistFull, Artist.PAINTING_ARRAY.count())
@@ -759,7 +760,7 @@ public class ColumnSelectIT extends ServerCase {
 
     @Test
     public void testAggregateColumnWithDisjointPrefetch() {
-        Property<Artist> artistFull = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistFull = PropertyFactory.createSelf(Artist.class);
 
         List<Object[]> result = ObjectSelect.query(Artist.class)
                 .columns(artistFull, Artist.PAINTING_ARRAY.count())
@@ -771,7 +772,7 @@ public class ColumnSelectIT extends ServerCase {
 
     @Test
     public void testAggregateColumnWithDisjointByIdPrefetch() {
-        Property<Artist> artistFull = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistFull = PropertyFactory.createSelf(Artist.class);
 
         List<Object[]> result = ObjectSelect.query(Artist.class)
                 .columns(artistFull, Artist.PAINTING_ARRAY.count())
@@ -791,14 +792,14 @@ public class ColumnSelectIT extends ServerCase {
 
             Object paintingsArr = artist.readPropertyDirectly(Artist.PAINTING_ARRAY.getName());
             assertFalse(paintingsArr instanceof Fault);
-            assertTrue(((List)paintingsArr).size() == (long)next[1]);
+            assertEquals(((List) paintingsArr).size(), (long) next[1]);
         }
     }
 
     @Test
     public void testObjectSelectWithJointPrefetch() {
         List<Artist> result = ObjectSelect.query(Artist.class)
-                .column(Property.createSelf(Artist.class))
+                .column(PropertyFactory.createSelf(Artist.class))
                 .prefetch(Artist.PAINTING_ARRAY.joint())
                 .select(context);
         assertEquals(20, result.size());
@@ -814,7 +815,7 @@ public class ColumnSelectIT extends ServerCase {
     @Test
     public void testObjectWithDisjointPrefetch() {
         List<Artist> result = ObjectSelect.query(Artist.class)
-                .column(Property.createSelf(Artist.class))
+                .column(PropertyFactory.createSelf(Artist.class))
                 .prefetch(Artist.PAINTING_ARRAY.disjoint())
                 .select(context);
         assertEquals(20, result.size());
@@ -829,7 +830,7 @@ public class ColumnSelectIT extends ServerCase {
     @Test
     public void testObjectWithDisjointByIdPrefetch() {
         List<Artist> result = ObjectSelect.query(Artist.class)
-                .column(Property.createSelf(Artist.class))
+                .column(PropertyFactory.createSelf(Artist.class))
                 .prefetch(Artist.PAINTING_ARRAY.disjointById())
                 .select(context);
         assertEquals(20, result.size());
@@ -847,10 +848,10 @@ public class ColumnSelectIT extends ServerCase {
 
     @Test
     public void testObjectColumn() {
-        Property<Artist> artistFull = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistProperty = PropertyFactory.createSelf(Artist.class);
 
         List<Object[]> result = ObjectSelect.query(Artist.class)
-                .columns(artistFull, Artist.ARTIST_NAME, Artist.PAINTING_ARRAY.count())
+                .columns(artistProperty, Artist.ARTIST_NAME, Artist.PAINTING_ARRAY.count())
                 .select(context);
         assertEquals(5, result.size());
 
@@ -864,8 +865,8 @@ public class ColumnSelectIT extends ServerCase {
 
     @Test
     public void testObjectColumnToOne() {
-        Property<Artist> artistFull = Property.create(ExpressionFactory.fullObjectExp(Painting.TO_ARTIST.getExpression()), Artist.class);
-        Property<Gallery> galleryFull = Property.create(ExpressionFactory.fullObjectExp(Painting.TO_GALLERY.getExpression()), Gallery.class);
+        EntityProperty<Artist> artistFull = PropertyFactory.createSelf(Painting.TO_ARTIST.getExpression(), Artist.class);
+        EntityProperty<Gallery> galleryFull = PropertyFactory.createSelf(Painting.TO_GALLERY.getExpression(), Gallery.class);
 
         List<Object[]> result = ObjectSelect.query(Painting.class)
                 .columns(Painting.PAINTING_TITLE, artistFull, galleryFull)
@@ -897,21 +898,25 @@ public class ColumnSelectIT extends ServerCase {
     }
 
     @Test
-    public void testObjectColumnToMany() throws Exception {
-        Property<Artist> artist = Property.createSelf(Artist.class);
+    public void testObjectColumnToMany() {
+        EntityProperty<Artist> artistProperty = PropertyFactory.createSelf(Artist.class);
 
         List<Object[]> result = ObjectSelect.query(Artist.class)
-                .columns(artist, Artist.PAINTING_ARRAY.flat(Painting.class), Artist.PAINTING_ARRAY.dot(Painting.TO_GALLERY))
+                .columns(artistProperty, Artist.PAINTING_ARRAY.flat(), Artist.PAINTING_ARRAY.dot(Painting.TO_GALLERY))
                 .select(context);
         assertEquals(21, result.size());
 
         for(Object[] next : result) {
             assertTrue(next[0] instanceof Artist);
-            assertTrue(next[1] instanceof Painting);
-            assertTrue(next[2] instanceof Gallery);
             assertEquals(PersistenceState.COMMITTED, ((Artist)next[0]).getPersistenceState());
-            assertEquals(PersistenceState.COMMITTED, ((Painting)(next[1])).getPersistenceState());
-            assertEquals(PersistenceState.COMMITTED, ((Gallery)(next[2])).getPersistenceState());
+            if(next[1] != null) {
+                assertTrue(next[1] instanceof Painting);
+                assertEquals(PersistenceState.COMMITTED, ((Painting)(next[1])).getPersistenceState());
+            }
+            if(next[2] != null) {
+                assertTrue(next[2] instanceof Gallery);
+                assertEquals(PersistenceState.COMMITTED, ((Gallery) (next[2])).getPersistenceState());
+            }
         }
     }
 
@@ -923,30 +928,36 @@ public class ColumnSelectIT extends ServerCase {
         assertEquals(21, result.size());
     }
 
-    @Test(expected = CayenneRuntimeException.class)
+    @Test
     public void testSelfPropertyInOrderBy() {
-        Property<Artist> artistProperty = Property.createSelf(Artist.class);
-        ObjectSelect.query(Artist.class)
+        EntityProperty<Artist> artistProperty = PropertyFactory.createSelf(Artist.class);
+        List<Artist> artists = ObjectSelect.query(Artist.class)
                 .column(artistProperty)
                 .orderBy(artistProperty.desc())
                 .select(context);
+        assertEquals(20, artists.size());
+        assertEquals("artist20", artists.get(0).getArtistName());
+        assertEquals("artist1", artists.get(19).getArtistName());
     }
 
-    @Test(expected = CayenneRuntimeException.class)
+    @Test
     public void testSelfPropertyInWhere() {
         Artist artist = ObjectSelect.query(Artist.class).selectFirst(context);
-        Property<Artist> artistProperty = Property.createSelf(Artist.class);
-        List<Artist> result = ObjectSelect.query(Artist.class)
+        EntityProperty<Artist> artistProperty = PropertyFactory.createSelf(Artist.class);
+        Artist selectedArtist = ObjectSelect.query(Artist.class)
                 .column(artistProperty)
                 .where(artistProperty.eq(artist))
-                .select(context);
+                .orderBy(artistProperty.asc())
+                .selectOne(context);
+        assertNotNull(selectedArtist);
+        assertEquals(artist.getArtistName(), selectedArtist.getArtistName());
     }
 
     @Test
     public void testObjPropertyInWhere() {
         Artist artist = ObjectSelect.query(Artist.class, Artist.ARTIST_NAME.eq("artist1"))
                 .selectFirst(context);
-        Property<Painting> paintingProperty = Property.createSelf(Painting.class);
+        EntityProperty<Painting> paintingProperty = PropertyFactory.createSelf(Painting.class);
         List<Painting> result = ObjectSelect.query(Painting.class)
                 .column(paintingProperty)
                 .where(Painting.TO_ARTIST.eq(artist))
@@ -1027,7 +1038,7 @@ public class ColumnSelectIT extends ServerCase {
     public void testNestedContextObjectResult() {
         ObjectContext childContext = runtime.newContext(context);
 
-        Property<Artist> artistProperty = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistProperty = PropertyFactory.createSelf(Artist.class);
         List<Artist> artists = ObjectSelect.columnQuery(Artist.class, artistProperty)
                 .select(childContext);
         assertEquals(20, artists.size());
@@ -1053,7 +1064,7 @@ public class ColumnSelectIT extends ServerCase {
     public void testNestedContextMixedResult() {
         ObjectContext childContext = runtime.newContext(context);
 
-        Property<Artist> artistProperty = Property.createSelf(Artist.class);
+        EntityProperty<Artist> artistProperty = PropertyFactory.createSelf(Artist.class);
         List<Object[]> data = ObjectSelect.columnQuery(Artist.class, Artist.ARTIST_NAME, artistProperty)
                 .select(childContext);
         assertEquals(20, data.size());
@@ -1080,4 +1091,125 @@ public class ColumnSelectIT extends ServerCase {
         assertArrayEquals(new byte[]{(byte)5, (byte)4, (byte)3, (byte)2}, blobs.get(1));
     }
 
+    @Test
+    public void testCollectionProperty() {
+        Painting painting = ObjectSelect.query(Painting.class).selectFirst(context);
+
+        Artist artist = ObjectSelect.query(Artist.class)
+                .where(Artist.PAINTING_ARRAY.contains(painting))
+                .and(Artist.DATE_OF_BIRTH.year().gt(1950))
+                .and(Artist.ARTIST_NAME.like("artist%"))
+                .selectOne(context);
+        assertNotNull(artist);
+        assertTrue(artist.getArtistName().startsWith("artist"));
+    }
+
+    @Test
+    public void test2PkSelect() {
+        List<Object[]> results = ObjectSelect.columnQuery(Artist.class,
+                PropertyFactory.createSelf(Artist.class),
+                PropertyFactory
+                        .createBase(ExpressionFactory
+                                .dbPathExp("paintingArray.toArtist.ARTIST_ID"),
+                                Integer.class))
+                .where(Artist.ARTIST_ID_PK_PROPERTY.eq(1L))
+                .pageSize(1)
+                .select(context);
+        assertEquals(1, results.size());
+        assertEquals("artist1", ((Artist)results.get(0)[0]).getArtistName());
+        assertEquals(1, results.get(0)[1]);
+    }
+
+    @Test
+    public void test2Objects2Pk() {
+        Artist artist = ObjectSelect.query(Artist.class)
+                .where(Artist.ARTIST_ID_PK_PROPERTY.eq(1L))
+                .selectFirst(context);
+        ArtistExhibit artistExhibit = context.newObject(ArtistExhibit.class);
+        Exhibit exhibit = context.newObject(Exhibit.class);
+        exhibit.setOpeningDate(new Date());
+        exhibit.setClosingDate(new Date());
+        artistExhibit.setToArtist(artist);
+        artistExhibit.setToExhibit(exhibit);
+        Gallery gallery = context.newObject(Gallery.class);
+        gallery.setGalleryName("Test");
+        exhibit.setToGallery(gallery);
+        context.commitChanges();
+
+        List<Object[]> results = ObjectSelect.columnQuery(Artist.class,
+                Artist.ARTIST_NAME,
+                PropertyFactory.createSelf(Artist.class),
+                PropertyFactory
+                        .createBase(ExpressionFactory
+                                        .dbPathExp("artistExhibitArray.ARTIST_ID"),
+                                Integer.class))
+                .where(Artist.ARTIST_ID_PK_PROPERTY.eq(1L))
+                .pageSize(1)
+                .select(context);
+        assertEquals(1, results.size());
+        assertEquals("artist1", results.get(0)[0]);
+        assertEquals("artist1", ((Artist)results.get(0)[1]).getArtistName());
+        assertEquals(1, results.get(0)[2]);
+    }
+
+    @Test
+    public void testMapToPojo() {
+        List<TestPojo> result = ObjectSelect.query(Artist.class)
+                .columns(Artist.ARTIST_NAME, Artist.DATE_OF_BIRTH, Artist.ARTIST_NAME.trim().length())
+                .where(Artist.ARTIST_NAME.like("artist%"))
+                .orderBy(Artist.ARTIST_ID_PK_PROPERTY.asc())
+                .map(TestPojo::new)
+                .select(context);
+
+        assertEquals(20, result.size());
+
+        TestPojo testPojo0 = result.get(0);
+        assertNotNull(testPojo0);
+        assertEquals("artist1", testPojo0.name);
+        assertNotNull(testPojo0.date);
+        assertEquals(7, testPojo0.length);
+
+        TestPojo testPojo19 = result.get(19);
+        assertNotNull(testPojo19);
+        assertEquals("artist20", testPojo19.name);
+        assertEquals(8, testPojo19.length);
+        assertNotNull(testPojo19.date);
+    }
+
+    @Test
+    public void testDoubleMapToPojo() {
+        List<TestPojo2> result = ObjectSelect.query(Artist.class)
+                .columns(Artist.ARTIST_NAME, Artist.DATE_OF_BIRTH, Artist.ARTIST_NAME.trim().length())
+                .where(Artist.ARTIST_NAME.like("artist%"))
+                .orderBy(Artist.ARTIST_ID_PK_PROPERTY.asc())
+                .map(TestPojo::new)
+                .map(TestPojo2::new)
+                .select(context);
+        assertEquals(20, result.size());
+
+        TestPojo2 testPojo0 = result.get(0);
+        assertNotNull(testPojo0);
+        assertEquals("artist1", testPojo0.pojo.name);
+        assertNotNull(testPojo0.pojo.date);
+        assertEquals(7, testPojo0.pojo.length);
+
+    }
+
+    static class TestPojo {
+        String name;
+        Date date;
+        int length;
+        TestPojo(Object[] data) {
+            name = (String)data[0];
+            date = (Date)data[1];
+            length = (Integer)data[2];
+        }
+    }
+
+    static class TestPojo2 {
+        TestPojo pojo;
+        TestPojo2(TestPojo pojo) {
+            this.pojo = pojo;
+        }
+    }
 }
